@@ -39,7 +39,7 @@ extern char *optarg;
 extern char *path_hash();
 extern char *prepath();
 extern char *strerror();
-extern char *sys_siglist[];
+extern char *strsignal();
 extern char *strchr();
 extern int errno;
 extern int eval();
@@ -1039,7 +1039,7 @@ sh_wait(argc, argv)
 		return 0;
 #if	defined(WNOHANG) && !defined(WLOBYTE)
 	if (status.w_termsig) {
-		fprintf(stderr, "%s", sys_siglist[status.w_termsig]);
+		fprintf(stderr, "%s", strsignam(status.w_termsig));
 		if (status.w_coredump)
 			fprintf(stderr, CORE_DUMPED);
 		fprintf(stderr, "\n");
@@ -1048,7 +1048,7 @@ sh_wait(argc, argv)
 		retcode = status.w_retcode;
 #else	/* !WNOHANG */
 	if ((status&0177) > 0) {
-		fprintf(stderr, "%s", sys_siglist[status&0177]);
+		fprintf(stderr, "%s", strsignal(status&0177));
 		if (status&0200)
 			fprintf(stderr, CORE_DUMPED);
 		fprintf(stderr, "\n");
