@@ -169,7 +169,7 @@ ShInit()
 		shsymbol[i].name = tLetter;
 		shsymbol[i].name2 = tSyntaxError;
 	}
-	for (cp = "0123456789"; *cp != '\0'; ++cp)
+	for (cp = (u_char*)"0123456789"; *cp != '\0'; ++cp)
 		shsymbol[*cp].name = tDigit;
 #if 0
 	for (cp = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"; *cp != '\0'; ++cp)
