@@ -131,7 +131,7 @@ reporterrs(cfpi)
 
 	/* exclusive access required, but we're the only scheduler... */
 	if ((fd = open(cfpi->mid, O_RDWR, 0)) < 0 ||
-	    (cfp = slurp(fd)) == NULL) {
+	    (cfp = slurp(fd, cfpi->id)) == NULL) {
 	  fprintf(stderr,
 		  "%s: unexpected absence of control file %s for error processing!\n",
 		  progname, cfpi->mid);
