@@ -98,8 +98,8 @@ extern int subsubdirs;
 extern int sweepinterval;
 extern struct config_entry *readconfig __((char *configfile));
 extern struct config_entry *rereadconfig __((struct config_entry *,char *));
-extern struct ctlfile *schedule __((int fd, char *file, struct config_entry *ce));
-extern struct ctlfile *slurp __((int fd)), *vtxprep __((struct ctlfile *));
+extern struct ctlfile *schedule __((int fd, char *file, struct config_entry *ce, long ino));
+extern struct ctlfile *slurp __((int fd, long ino)), *vtxprep __((struct ctlfile *));
 extern time_t time();
 extern u_char *rfc822date __((time_t *));
 extern void deletemsg __((char *, struct ctlfile *));
@@ -121,6 +121,7 @@ extern void defaultconfigentry __((struct config_entry *ce, struct config_entry
 
 struct dirstatname {
 	struct stat st;
+	long ino;
 	char name[1]; /* Allocate enough size */
 };
 struct dirqueue {
@@ -315,10 +316,11 @@ main(argc, argv)
 	if (optind < argc) {
 	  /* process the specified control files only */
 	  for (; optind < argc; ++optind) {
+	    long ino = atol(argv[optind]);
 	    if ((fd = eopen(argv[optind], O_RDWR, 0)) < 0)
 	      continue;
 	    /* the close(fd) is done in vtxprep */
-	    cfp = schedule(fd, argv[optind], tp);
+	    cfp = schedule(fd, argv[optind], tp, ino);
 	    if (cfp == NULL) {
 	      if (verbose)
 		fprintf(stderr, "Nothing scheduled for %s!\n",
@@ -479,8 +481,7 @@ const void *b;
 	return ((*A)->st.st_ctime - (*B)->st.st_ctime);
 }
 
-int
-dirqueuescan(dir, dq)
+static int dirqueuescan(dir, dq)
 	const char *dir;
 	struct dirqueue *dq;
 {
@@ -510,34 +511,18 @@ dirqueuescan(dir, dq)
 	  if (dp->d_name[0] >= '0' &&
 	      dp->d_name[0] <= '9') {
 	    /* A file whose name STARTS with a number (digit) */
-	    strcpy(file, dp->d_name);
 
-	    if (lstat(file,&stbuf) != 0 ||
-		!S_ISREG(stbuf.st_mode)) {
-	      /* Let it be there for the manager to wonder.. */
-	      /*
-	       * This also avoids a race when the
-	       * scheduler starts up while the router
-	       * is pushing stuff at it, and at first
-	       * links to the scheduler, and then to
-	       * the transport dir..
-	       *
-	       * We propably need to check on this (directory)
-	       * latter to see if the file appeared also to the
-	       * transport dir.
-	       */
-	      continue;
-	    }
-
-	    if (in_dirscanqueue(dq,stbuf.st_ino))
+	    long ino = atol(dp->d_name);
+	    if (in_dirscanqueue(dq,ino))
 	      /* Already in pre-schedule-queue */
 	      continue;
 
-	    /* If we are re-syncing in from the $POSTOFFICE/transports/
-	       directory, we may have this file in processing state...  */
+	    strcpy(file, dp->d_name);
+
+	    /* We may have this file in processing state...  */
 	    {
 	      struct spblk *spl;
-	      spl = sp_lookup((long)stbuf.st_ino, spt_mesh[L_CTLFILE]);
+	      spl = sp_lookup(ino, spt_mesh[L_CTLFILE]);
 	      if (spl != NULL) {
 		/* Already in processing, don't touch.. */
 		/*printf("File: %s active (not previously locked)\n",file);*/
@@ -546,9 +531,18 @@ dirqueuescan(dir, dq)
 	      }
 	    }
 
+
+	    if (lstat(file,&stbuf) != 0 ||
+		!S_ISREG(stbuf.st_mode)) {
+	      /* Not a regular file.. Let it be there for the manager
+		 to wonder.. */
+	      continue;
+	    }
+
 	    /* Now store the entry */
 	    dsn = (struct dirstatname*)emalloc(sizeof(*dsn)+strlen(file));
 	    memcpy(&(dsn->st),&stbuf,sizeof(stbuf));
+	    dsn->ino = ino;
 	    strcpy(dsn->name,file);
 
 	    if (dq->wrkspace <= dq->wrkcount) {
@@ -597,6 +591,7 @@ syncweb(dq, cehead)
 	char *file;
 	struct spblk *spl;
 	int wrkcnt = 0;
+	long ino;
 
 	/* Any work to do ? */
 	if (dq->wrkcount == 0) return 0;
@@ -605,13 +600,14 @@ syncweb(dq, cehead)
 	dq->wrkcount -= 1;
 	file  =   dq->stats[dq->wrkcount]->name;
 	stbuf = &(dq->stats[dq->wrkcount]->st);
+	ino   =   dq->stats[dq->wrkcount]->ino;
 	/* Now we have pointers */
 
 	/* Sometimes we may get files already in processing
 	   into our pre-schedule queue */
 
 	/* Already in processing ? */
-	spl = sp_lookup((long)stbuf->st_ino, spt_mesh[L_CTLFILE]);
+	spl = sp_lookup(ino, spt_mesh[L_CTLFILE]);
 	if (spl == NULL) {
 	  /* Not yet in processing! */
 	  int fd;
@@ -622,7 +618,7 @@ syncweb(dq, cehead)
 	      eunlink(file);	/* hrmpf! */
 	  } else {
 	    /* Ok, schedule! */
-	    schedule(fd, file, cehead);
+	    schedule(fd, file, cehead, ino);
 	    ++wrkcnt;
 	  }
 	}
@@ -643,16 +639,17 @@ syncweb(dq, cehead)
  * done at the appropriate time in the future.
  */
 struct ctlfile *
-schedule(fd, file, cehead)
+schedule(fd, file, cehead, ino)
 	int fd;
 	char *file;
 	struct config_entry *cehead;
+	long ino;
 {
 	struct ctlfile *cfp;
 	struct vertex *vp;
 
 	/* read and process the control file */
-	if ((cfp = vtxprep(slurp(fd))) == NULL) {
+	if ((cfp = vtxprep(slurp(fd, ino))) == NULL) {
 	  if (!vtxprep_skip) {	/* Unless skipped.. */
 	    eunlink(file);	/* everything here has been processed */
 	    if (verbose)
@@ -674,8 +671,9 @@ schedule(fd, file, cehead)
 }
 
 struct ctlfile *
-slurp(fd)
+slurp(fd, ino)
 	int fd;
+	long ino;
 {
 	register char *s;
 	register int i;
@@ -765,7 +763,7 @@ slurp(fd)
 	/* copy over the offsets */
 	while (--i >= 0)
 	  *lp++ = *ip++;
-	cfp->id = stbuf.st_ino;
+	cfp->id  = ino;
 	cfp->mid = NULL;
 	free(offset);	/* release the block */
 	return cfp;
@@ -1445,21 +1443,3 @@ file_to_ctl(s)
 	  return NULL;
 	return (struct ctlfile *)spl->data;
 }
-
-/*
- * time-string to be prepended on logged messages
- */
-char *
-timestring()
-{
-	static char timebuf[40];
-	time_t now;
-	struct tm *tp;
-
-	time(&now);
-	tp = localtime(&now);
-	sprintf(timebuf,"%d%02d%02d%02d%02d%02d",
-		tp->tm_year+1900, tp->tm_mon+1, tp->tm_mday,
-		tp->tm_hour, tp->tm_min, tp->tm_sec);
-	return timebuf;
-}
