@@ -65,9 +65,9 @@ struct ctlfile {
 	struct vertex	*head;		/* head of the list of groups */
 	struct vertex	*atspool;	/* the vertex currently scheduled */
 	int	nlines;		/* number of lines/entries in the file */
+	char	*erroraddr;	/* error address(es) */
 	char	*contents;	/* the control file as copied into memory */
 	char	*logident;	/* identification for log entries */
-	char	*erroraddr;	/* error address(es) */
 	char	*deliveryform;	/* [mea] Name of delivery error form */
 	char	*envid;		/* DSN envid */
 	u_long	id;		/* identification # (inode# of control file) */
@@ -88,12 +88,14 @@ struct web {
 };
 
 struct procinfo {
-	pid_t	pid;
+	pid_t	pid;		/* Process-id				*/
 	int	reaped;
-	struct web *ch;
-	struct web *ho;
-	struct vertex *vertices;
-	char	*carryover;
+	int	tofd;		/* tell transporter job data thru this	*/
+	int	hungry;		/* State of reports from child		*/
+	struct web *ch;		/* Web of CHANNELs			*/
+	struct web *ho;		/* Web of HOSTs				*/
+	struct vertex *vertices; /* Vertixes that are yet to be sent	*/
+	char	*carryover;	/* Long responces..			*/
 };
 
 /* Stores the offset indices of all addresses that have same channel and host */
@@ -118,6 +120,7 @@ struct vertex {
 	int		attempts;	/* count of number of TA invocations */
 	int		retryindex;	/* cur index into ce->retries array */
 	time_t		wakeup;		/* time to wake up and run this */
+	char		*sender;	/* Message Sender/error recipient */
 	int		ngroup;		/* number of addresses in group */
 	int		index[1];	/* index of cfp->offset for group */
 };
