@@ -8,6 +8,7 @@
 #include <sys/param.h>
 #include "hostenv.h"
 #include "scheduler.h"
+#include "mail.h"
 #include <fcntl.h>
 #include <sys/stat.h>
 #include <errno.h>
@@ -46,11 +47,12 @@ extern int subsubdirs;
 extern int verbose;
 extern time_t qipcretry, time();
 extern void dotransport();
-extern void qprint(), readfrom();
+extern void qprint();
+static void readfrom __((int fd));
 extern void runcommand();
 extern void stashprocess();
 extern void unweb(), reschedule(), fifoschedule();
-extern void update();
+extern void update __((int fd, char *diags));
 
 
 /*
@@ -86,40 +88,10 @@ tscan(spl)
 	  /* cfp->atspool = NULL; */
 	  return -1;
 	}
-#if 1
-	{
-	  char *dir, path[MAXPATHLEN], buf[MAXPATHLEN];
 
-	  /*
-	   * XX: Note that any old filename will do for the control file,
-	   * since all names are links to the same file anyway.
-	   * However, we need to get the filename right, based on
-	   * our favorite filename generation technique.
-	   *
-	   * XX: Wrong, not just ANY, it MUST match the thing each
-	   *     transporter expects!  Thus when all hosts become
-	   *     processed, the links can be released.
-	   *
-	   * Solution: We use  cfp->atspool ((vertex)) ->linkpath  for it!
-	   */
-
-	  if (subsubdirs)	/* channel/host */
-	    dir = vtx_to_dir(cfp->atspool, buf);
-	  else		/* channel */
-	    dir = cfp->atspool->orig[L_CHANNEL]->name;
-	  sprintf(path, "%s/%lu", dir, cfp->id);
-
-	  if (verbose)
-	    printf("  tscan() path=\"%s\"\n",path);
-	  t_files[t_n++] = path;
-	}
-#else
-	if (verbose)
-	  printf("  tscan() path=\"%s\"\n",cfp->atspool->linkpath);
-
-	t_files[t_n++] = cfp->atspool->linkpath;
-#endif
+	t_files[t_n++] = cfp->mid;
 	cfp->mark    = V_NONE;
+
 	/* cfp->atspool = NULL; */
 	return 0;
 }
@@ -142,12 +114,6 @@ transport(vhead, channel, host)
 
 	sp_scan(tscan, (struct spblk *)NULL, spt_mesh[L_CTLFILE]);
 	dotransport(files, t_n, vhead, channel, host);
-
-#if 0
-	/* Free the strings malloced on  tscan() */
-	for (i = 0; i < t_n; ++i)
-	  free(files[i]);
-#endif
 }
 
 void
@@ -258,7 +224,7 @@ runcommand(filearr, nelem, argv, vhead, chwp, howp)
 	int	i, pid, position, to[2], from[2], uid, gid;
 	char	*cmd;
 	FILE	*fp;
-	static int pipesize;
+	static int pipesize = 0;
 
 
 	uid = vhead->ce.uid;
@@ -290,7 +256,8 @@ runcommand(filearr, nelem, argv, vhead, chwp, howp)
 	    dup2(from[1],1);	/* child stdout */
 	    close(from[1]);
 	  }
-	  dup2(1,2);
+	  /* dup2(1,2); */
+
 	  /* keep current stderr for child stderr */
 	  /* close all other open filedescriptors */
 #if	1
@@ -323,7 +290,7 @@ runcommand(filearr, nelem, argv, vhead, chwp, howp)
 	  if (chwp != NULL) chwp->kids += 1;
 	  if (howp != NULL) howp->kids += 1;
 	  /* save from[0] away as a descriptor to watch */
-	  stashprocess(pid, from[0], chwp, howp, vhead);
+	  stashprocess(pid, from[0], to[1], chwp, howp, vhead);
 	  /* write the list of files into to[1], and close it */
 	  if ((fp = fdopen(to[1], "w")) == NULL) {
 	    if (verbose)
@@ -348,16 +315,16 @@ runcommand(filearr, nelem, argv, vhead, chwp, howp)
 }
 
 void
-stashprocess(pid, fd, chwp, howp, vhead)
-	int pid, fd;
+stashprocess(pid, fromfd, tofd, chwp, howp, vhead)
+	int pid, fromfd, tofd;
 	struct web *chwp, *howp;
 	struct vertex *vhead;
 {
 	int i;
 
 	if (verbose)
-	  fprintf(stderr, "stashprocess(%d, %d, %s, %s)\n",
-		  pid, fd, chwp ? chwp->name : "nil",
+	  fprintf(stderr, "stashprocess(%d, %d, %d, %s, %s)\n",
+		  pid, fromfd, tofd, chwp ? chwp->name : "nil",
 		  howp ? howp->name : "nil");
 	if (cpids == NULL) {
 	  if (scheduler_nofiles < 1)
@@ -368,18 +335,20 @@ stashprocess(pid, fd, chwp, howp, vhead)
 	  while (--i >= 0)
 	    cpids[i].pid = 0;
 	}
-	cpids[fd].pid    = pid;
-	cpids[fd].reaped = 0;
-	cpids[fd].ch     = chwp;
-	cpids[fd].ho     = howp;
-	cpids[fd].vertices = vhead;
-	vhead->proc = &cpids[fd];
-	cpids[fd].carryover = NULL;
+	cpids[fromfd].pid    = pid;
+	cpids[fromfd].reaped = 0;
+	cpids[fromfd].ch     = chwp;
+	cpids[fromfd].ho     = howp;
+	cpids[fromfd].vertices = vhead;
+	cpids[fromfd].tofd   = tofd;
+	vhead->proc = &cpids[fromfd];
+	cpids[fromfd].carryover = NULL;
+	cpids[fromfd].hungry = 1;
 #ifdef O_NONBLOCK /* POSIXy thing */
 	/* set up non-blocking I/O */
-	i = fcntl(fd, F_GETFL, 0);
+	i = fcntl(fromfd, F_GETFL, 0);
 	i |= O_NONBLOCK;
-	i = fcntl(fd, F_SETFL, i);
+	i = fcntl(fromfd, F_SETFL, i);
 #else
 #ifdef	USE_BSDNODELAY
 #ifdef	FNONBLOCK
@@ -387,49 +356,53 @@ stashprocess(pid, fd, chwp, howp, vhead)
 #else
 	i |= FNDELAY;
 #endif
-	i = fcntl(fd, F_SETFL, i);
+	i = fcntl(fromfd, F_SETFL, i);
 #endif	/* USE_BSDNODELAY */
 #endif	/* O_NONBLOCK  */
 }
 
 void
-reclaim(fd)
-	int fd;
+reclaim(fromfd)
+	int fromfd;
 {
 	struct vertex *vp, *nvp;
 
-	cpids[fd].pid = 0;
-	cpids[fd].reaped = 0;
-	if (cpids[fd].carryover != NULL) {
+	cpids[fromfd].pid = 0;
+	cpids[fromfd].reaped = 0;
+	if (cpids[fromfd].carryover != NULL) {
 	  fprintf(stderr, "%s: HELP! Lost %d bytes: '%s'\n",
-		  progname, strlen(cpids[fd].carryover),
-		  cpids[fd].carryover);
-	  free(cpids[fd].carryover);
-	  cpids[fd].carryover = NULL;
+		  progname, strlen(cpids[fromfd].carryover),
+		  cpids[fromfd].carryover);
+	  free(cpids[fromfd].carryover);
+	  cpids[fromfd].carryover = NULL;
 	}
-	if (cpids[fd].ch != NULL) {
-	  --cpids[fd].ch->kids;
-	  if (cpids[fd].ch->fifohead)
-	    fifoschedule(cpids[fd].ch);
-	  else if (cpids[fd].ch->kids == 0 && cpids[fd].ch->link == NULL) {
-	    unweb(L_CHANNEL, cpids[fd].ch);
-	    cpids[fd].ch = NULL;
+	if (cpids[fromfd].ch != NULL) {
+	  --cpids[fromfd].ch->kids;
+	  if (cpids[fromfd].ch->fifohead)
+	    fifoschedule(cpids[fromfd].ch);
+	  else if (cpids[fromfd].ch->kids == 0 &&
+		   cpids[fromfd].ch->link == NULL) {
+	    unweb(L_CHANNEL, cpids[fromfd].ch);
+	    cpids[fromfd].ch = NULL;
 	  }
 	}
-	if (cpids[fd].ho != NULL) {
-	  --cpids[fd].ho->kids;
-	  if (cpids[fd].ho->fifohead)
-	    fifoschedule(cpids[fd].ho);
-	  else if (cpids[fd].ho->kids == 0 && cpids[fd].ho->link == NULL) {
-	    unweb(L_HOST, cpids[fd].ho);
-	    cpids[fd].ho = NULL;
+	if (cpids[fromfd].ho != NULL) {
+	  --cpids[fromfd].ho->kids;
+	  if (cpids[fromfd].ho->fifohead)
+	    fifoschedule(cpids[fromfd].ho);
+	  else if (cpids[fromfd].ho->kids == 0 &&
+		   cpids[fromfd].ho->link == NULL) {
+	    unweb(L_HOST, cpids[fromfd].ho);
+	    cpids[fromfd].ho = NULL;
 	  }
 	}
-	close(fd);
+	close(fromfd);
 
-	/* Reschedule the vertices that are left (that were not reported on). */
-	nvp = cpids[fd].vertices;
-	cpids[fd].vertices = NULL;
+	/* Reschedule the vertices that are left
+	   (that were not reported on).		*/
+
+	nvp = cpids[fromfd].vertices;
+	cpids[fromfd].vertices = NULL;
 	for (vp = nvp; vp != NULL ; vp = nvp) {
 	  nvp = vp->nextitem;
 	  if (vp->ngroup == 0)
@@ -535,20 +508,23 @@ typedef	struct fd_set { fd_mask	fds_bits[1]; } fd_set;
 #endif	/* !FD_SET */
 #endif	/* !NFDBITS */
 
-void
-mux(quickcheck)
-int quickcheck;
+int
+mux(timeout)
+time_t timeout;
 {
 	int	i, n, rc, maxf;
 	fd_set	mask;
 	struct sockaddr_in raddr;
 	int	raddrlen;
 	struct timeval tv;
+	time_t now;
 
-	tv.tv_sec = 10000; /* LONG delay.. */
+	time(&now);
+
+	tv.tv_sec = timeout - now;
+	if (timeout < now)
+	  tv.tv_sec = 0;
 	tv.tv_usec = 0;
-	if (quickcheck)
-	  tv.tv_sec = 0; /* Check the sockets fast.. */
 
 	maxf = -1;
 	FD_ZERO(&mask);
@@ -564,18 +540,18 @@ int quickcheck;
 	    maxf = querysocket;
 	}
 	if (maxf < 0)
-	  return;
+	  return -1;
 	++maxf;
 	/*fprintf(stderr, "about to select on %x [%d]\n",
 			  mask.fds_bits[0], maxf); */
 	if ((n = select(maxf, &mask, NULL, NULL, &tv)) < 0) {
 	  /* fprintf(stderr, "got an interrupt (%d)\n", errno); */
 	  if (errno == EINTR)
-	    return;
+	    return 0;
 	} else if (n == 0) {
 	  /* fprintf(stderr, "abnormal 0 return from select!\n"); */
 	  /* -- just a timeout -- fast or long */
-	  return;
+	  return 1;
 	} else {
 	  /*fprintf(stderr, "got %d ready (%x)\n", n, mask.fds_bits[0]);*/
 	  if (querysocket >= 0 && FD_ISSET(querysocket, &mask)) {
@@ -609,6 +585,7 @@ int quickcheck;
 	    }
 	}
 	/* fprintf(stderr, "return from mux\n"); */
+	return 0;
 }
 
 #if 0 /* ipcpause() NOT in use, all done now inside  mux() */
@@ -709,11 +686,12 @@ queryipcinit()
 #endif	/* AF_INET */
 }
 #else	/* !HAVE_SELECT */
-void
-mux(quickcheck)
-int quickcheck; /* Is always a quickcheck..  Aurgh the induced load.. */
+int
+mux(timeout)
+time_t timeout;
 {
 	int	fd;
+	time_t	now;
 
 	/*
 	 * Nice'n easy and simpleminded: grab a random file descriptor,
@@ -726,6 +704,11 @@ int quickcheck; /* Is always a quickcheck..  Aurgh the induced load.. */
 	  for (fd = 0; fd < scheduler_nofiles ; ++fd)
 	    if (cpids[fd].pid != 0)
 	      readfrom(fd);
+
+	time(&now);
+	if (timeout > now)
+	  sleep(1);
+	return 1;
 }
 
 ipcpause(pollflag)
@@ -741,7 +724,7 @@ queryipcinit()
 }
 #endif	/* HAVE_SELECT */
 
-void
+static void
 readfrom(fd)
 	int fd;
 {
@@ -781,7 +764,7 @@ readfrom(fd)
 	      if (verbose)
 		fprintf(stderr, "%d processed: %s\n",
 			fd, pcp);
-	      update(pcp);
+	      update(fd,pcp);
 	      *cp = '_';
 	      pcp = cp + 1;
 	      dontbreak = 0;
