@@ -77,10 +77,11 @@
 extern int optind;
 extern char *optarg;
 
-#ifndef __STDC__
 extern char *malloc();
+
+#ifndef __STDC__
 extern FILE *fopen();
-extern char *sprintf();
+/* extern char *sprintf(); */
 extern char *strcpy();
 extern int atoi();
 extern int fprintf();
@@ -88,7 +89,7 @@ extern int getopt();
 extern int printf();
 extern int strcmp();
 extern int strlen();
-extern int tolower();
+/* extern int tolower(); */
 extern void abort();
 extern void exit();
 #endif
@@ -689,11 +690,13 @@ FILE *entryfp;
 
 char *progname;
 
+#ifndef __
 #ifdef __STDC__
 # define __(x) x
 #else
 # define __(x) ()
 #endif
+#endif
 
 extern char *basename __((char *)); 
 extern char *strsave __((char *));
