@@ -268,7 +268,7 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	  return NULL;
 	}
 
-	d.ctlid = stbuf.st_ino;
+	d.ctlid = atol(file);
 	d.senders = NULL;
 	d.recipients = NULL;
 	d.logident = "none";
