@@ -16,6 +16,8 @@
 
 
 extern char *strdup();
+extern void *emalloc();
+extern void *erealloc();
 
 char *notarybuf = NULL;
 
