@@ -502,6 +502,9 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	else
 	  writeheaders(startrp, tafp, "\n",   convertmode, maxwidth);
 
+	if (verboselog)
+	  writeheaders(startrp, verboselog, "\n", convertmode, maxwidth);
+
 	/* append message body itself */
 	if ((i = appendlet(dp, mp, tafp, verboselog, convertmode)) != EX_OK) {
 	  for (rp = startrp; rp != endrp; rp = rp->next)
