@@ -41,8 +41,8 @@
 #include <sys/param.h>
 #include <sys/stat.h>
 #include NDIR_H
-#ifdef USE_ALLOCA
-# include <alloca.h>
+#if defined(USE_ALLOCA) && defined(ALLOCA_H)
+# include ALLOCA_H
 #endif
 
 #include <netdb.h>
@@ -191,7 +191,7 @@ int force_8bit = 0;		/* Claim to the remote to be 8-bit system, even
 				   when it doesn't report itself as such..*/
 int force_7bit = 0;		/* and reverse the previous.. */
 int checkwks = 0;
-FILE *logfp;
+FILE *logfp = NULL;
 extern int nobody;
 int daemon_uid = -1;
 int first_uid = 0;		/* Make the opening connect with the UID of the
@@ -227,6 +227,8 @@ extern void warning(), report();
 #ifndef MALLOC_TRACE
 extern univptr_t emalloc();
 #endif
+extern int  atoi __((char*));
+extern long atol __((char*));
 extern char *strerror();
 extern char *strchr();
 extern char *strrchr();
@@ -242,7 +244,8 @@ extern int smtpconn __((char *host, FILE *fparr[], FILE *verboselog));
 extern int smtp_ehlo __((FILE *smtpfp[], FILE *verboselog, char *strbuf));
 extern int ehlo_check __((char *buf));
 extern int smtpwrite __((FILE *smtpfp[2], FILE *verboselog, int saverpt, char *linebuf));
-extern int emptyline(), process();
+extern int emptyline();
+extern int process __((struct ctldesc*,int,char*,FILE**,FILE*));
 extern void diagnostic();
 extern void notaryreport __((char *rcpt, char *acct, char *status));
 extern void notarystatsave __((char *smtpstatline));
@@ -254,23 +257,28 @@ extern int check_7bit_cleanness __((struct ctldesc *dp));
 
 extern void prversion();
 extern void getnobody();
-extern int makeconn();
-extern int vcsetup();
-extern void hp_init();
-extern char **hp_getaddr(), **hp_nextaddr();
-extern void hp_setalist();
+extern int makeconn __((struct hostent*, int*, FILE*, FILE**,int));
+extern int vcsetup __((char*,int,int*,char*,FILE*));
+extern void hp_init __((struct hostent*));
+extern char **hp_getaddr __((void)), **hp_nextaddr __((void));
+extern void hp_setalist __((struct hostent*,char**));
 #ifdef	BIND
-extern int rightmx();
+extern int rightmx __((char*,char*));
 extern int h_errno;
 extern int res_mkquery(), res_send(), dn_skipname(), dn_expand();
 # ifdef RFC974
-extern int getrr(), getmxrr();
-extern int getrrtype();
+extern int getrr __((char*,int,u_short,int));
+struct mxdata {
+	msgdata	*host;
+	u_short	pref;
+};
+extern int getmxrr __((char*,struct mxdata*,int,int,int*));
+extern int getrrtype __((char*,int,u_short,int));
 # endif /* RFC974 */
 #endif	/* BIND */
-extern int matchroutermxes();
-extern SIGNAL_TYPE sig_pipe();
-extern SIGNAL_TYPE sig_alarm();
+extern int matchroutermxes __((char*,struct address*));
+extern SIGNAL_TYPE sig_pipe __((int));
+extern SIGNAL_TYPE sig_alarm __((int));
 extern int getmyhostname();
 extern void stashmyaddresses();
 extern int cistrcmp(), cistrncmp();
@@ -429,7 +437,7 @@ main(argc, argv)
 	  else
 	    logfp = (FILE *)fdopen(fd, "a");
 	} else
-		logfp = NULL;
+	  logfp = NULL;
 
 	getnobody();
 	getdaemon();
@@ -461,7 +469,7 @@ main(argc, argv)
 		fclose(smtpfp[0]); smtpfp[0] = NULL;
 		fclose(smtpfp[0]); smtpfp[0] = NULL;
 		if (logfp)
-		  fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
+		  fprintf(logfp, "%d#\t(closed SMTP channel - new host)\n", pid);
 		smtp_isopen = 0;
 	      }
 	    }
@@ -514,7 +522,7 @@ main(argc, argv)
 	  fclose(smtpfp[0]);
 	  fclose(smtpfp[1]);
 	  if (logfp)
-	    fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
+	    fprintf(logfp, "%d#\t(closed SMTP channel - final close)\n", pid);
 	}
 	smtp_isopen = 0;
 
@@ -571,7 +579,7 @@ process(dp, smtpstatus, host, smtpfp, verboselog)
 		 fclose(smtpfp[0]); smtpfp[0] = NULL;
 		 fclose(smtpfp[1]); smtpfp[1] = NULL;
 		 if (logfp)
-		 fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
+		 fprintf(logfp, "%d#\t(closed SMTP channel - after delivery failure)\n", pid);
 		 smtp_isopen = 0;
 		 }
 	       */
@@ -630,7 +638,7 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	  switch (content_kind) {
 	  case 0:		/* Not MIME */
 	    if ((ehlo_capabilities & ESMTP_8BITMIME) == 0 &&
-		!ascii_clean) {
+		!ascii_clean && !force_8bit) {
 	      convertmode = _CONVERT_UNKNOWN;
 	      downgrade_headers(startrp, convertmode, verboselog);
 	    }
@@ -675,7 +683,7 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	   adding msg-body size to the largest known header size,
 	   though excluding possible header and body rewrites.. */
 	if (ehlo_capabilities & ESMTP_SIZEOPT) {
-	  sprintf(s," SIZE=%d",startrp->desc->msgsizeestimate);
+	  sprintf(s," SIZE=%ld",startrp->desc->msgsizeestimate);
 	  s += strlen(s);
 	}
 	/* DSN parameters ... */
@@ -787,9 +795,6 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	else
 	  size = -hsize;
 
-	if (lseek(dp->msgfd, dp->msgbodyoffset, SEEK_SET) < 0L)
-	  warning("Cannot seek to message body! (%m)", (char *)NULL);
-
 	if ((r = appendlet(dp, smtpfp[1], verboselog,
 			   hsize, size, convertmode)) != EX_OK) {
 	fail:
@@ -807,7 +812,7 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	    fclose(smtpfp[0]); smtpfp[0] = NULL;
 	    fclose(smtpfp[0]); smtpfp[0] = NULL;
 	    if (logfp)
-	      fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
+	      fprintf(logfp, "%d#\t(closed SMTP channel - appendlet() failure)\n", pid);
 	    smtp_isopen = 0;
 	  }
 	  return EX_TEMPFAIL;
@@ -869,8 +874,7 @@ appendlet(dp, fp, verboselog, hsize, msize, convertmode)
 
 	register int i, rc;
 	register int bufferfull;
-	int position = dp->msgbodyoffset;
-	int lastwasnl;
+	int lastwasnl = 0;
 	int lastch = '\n'; /* WriteMIMELine() can decode-QP and then the
 			      "lastwasnl" is no longer valid .. */
 
@@ -893,16 +897,16 @@ appendlet(dp, fp, verboselog, hsize, msize, convertmode)
 	    }
 	    return EX_IOERR;
 	  }
-	  if (let_buffer[readalready-1] != '\n')
-	    writebuf(fp, "\n", 1);
+	  lastwasnl = (let_buffer[readalready-1] != '\n')
 	  if (statusreport)
 	    report("DATA %d (100%%)", readalready+hsize);
 	  if (fflush(fp) != 0)
 	    return EX_IOERR;
-	  return EX_OK;
 	}
 #endif
-	/* we are assumed to be positioned properly at start of message body */
+	if (lseek(dp->msgfd, dp->msgbodyoffset, SEEK_SET) < 0L)
+	  warning("Cannot seek to message body! (%m)", (char *)NULL);
+
 	bufferfull = 0;
 	lastwasnl = 1;	/* we are guaranteed to have a \n after the header */
 	if (convertmode == _CONVERT_NONE) {
@@ -951,7 +955,7 @@ appendlet(dp, fp, verboselog, hsize, msize, convertmode)
 #define MFPCLOSE i = dup(dp->msgfd); fclose(mfp); dup2(i,dp->msgfd); close(i);
 	  bufferfull = 0;
 #else /* USE_MMAP */
-	  char *s2, *s = dp->let_buffer + dp->msgbodyoffset;
+	  char *s = dp->let_buffer + dp->msgbodyoffset;
 
 #define MFPCLOSE
 #endif
@@ -1338,7 +1342,7 @@ smtpopen(host, fparr, verboselog)
 	      fclose(fparr[0]); fparr[0] = NULL;
 	      fclose(fparr[1]); fparr[1] = NULL;
 	      if (logfp)
-		fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
+		fprintf(logfp, "%d#\t(closed SMTP channel - HELO failed??)\n", pid);
 	      smtp_isopen = 0;
 	    }
 	  }
@@ -1350,11 +1354,6 @@ smtpopen(host, fparr, verboselog)
 	return i;
 }
 
-struct mxdata {
-	msgdata	*host;
-	u_short	pref;
-};
-
 int
 smtpconn(host, fparr, verboselog)
 	char *host;
@@ -1583,7 +1582,7 @@ makeconn(hp, fdp, verboselog, fparr, mxcnt)
 	}
 #endif	/* RFC974 */
 #endif	/* BIND */
-	
+
 	retval = EX_UNAVAILABLE;
 	for (hp_init(hp); *hp_getaddr() != NULL; hp_nextaddr()) {
 	  if (matchmyaddress(*hp_getaddr(),hp->h_length)) {
@@ -1591,38 +1590,46 @@ makeconn(hp, fdp, verboselog, fparr, mxcnt)
 	    sprintf(remotemsg,"Trying to talk with myself!");
 	    break;		/* TEMPFAIL or UNAVAILABLE.. */
 	  }
+
+	  if (fparr[0] != NULL || fparr[1] != NULL) {
+	    /* Clean (close) these fds -- they have noted to leak.. */
+	    if (fparr[0] != NULL) fclose(fparr[0]); fparr[0] = NULL;
+	    if (fparr[1] != NULL) fclose(fparr[1]); fparr[1] = NULL;
+	    if (logfp)
+	      fprintf(logfp,"%d#\t(closed SMTP channel at makeconn(), isopen=%d)\n",pid,smtp_isopen);
+	    smtp_isopen = 0;
+	  }
+
 	  switch (vcsetup(*hp_getaddr(), hp->h_length, fdp, hostbuf, verboselog)) {
 	  case EX_OK:
-	    /* Clean (close) these fds -- they have noted to leak.. */
-	    if (fparr[0] != NULL) fclose(fparr[0]);
-	    if (fparr[1] != NULL) fclose(fparr[1]);
-
-	    if ((fparr[0] = fdopen(*fdp, "r")) == NULL) {
-	      int err;
+	      if ((fparr[0] = fdopen(*fdp, "r")) == NULL) {
+		int err;
 fdopen_fail:
-	      err = errno;
-	      if (fparr[0] != NULL) fclose(fparr[0]);
-	      if (fparr[1] != NULL) fclose(fparr[1]);
-printf("smtp: Failed to fdopen() a socket stream, errno=%d, err='%s' Hmm ??\n",err, strerror(err));
-fflush(stdout);
-abort();
-	    }
-	    if ((fparr[1] = fdopen(*fdp, "w")) == NULL)
-	      goto fdopen_fail;
+		err = errno;
+		if (fparr[0] != NULL) fclose(fparr[0]); fparr[0] = NULL;
+		if (fparr[1] != NULL) fclose(fparr[1]); fparr[1] = NULL;
+		printf("smtp: Failed to fdopen() a socket stream, errno=%d, err='%s' Hmm ??\n",err, strerror(err));
+		fflush(stdout);
+		abort();
+	      }
+	      if ((fparr[1] = fdopen(*fdp, "w")) == NULL) {
+		goto fdopen_fail;
+	      }
 
-	    if (esmtp_on_banner > 0)
-	      esmtp_on_banner = 0;
-	    retval = smtpwrite(fparr, verboselog, 1, NULL);
-	    if (retval != EX_OK)
-	      /*
-	       * If you want to continue with the next host,
-	       * the below should be 'return EX_TEMPFAIL'.
-	       */
-	      break;		/* try another host address */
-	    return EX_OK;
+	      if (esmtp_on_banner > 0)
+		esmtp_on_banner = 0;
+
+	      retval = smtpwrite(fparr, verboselog, 1, NULL);
+	      if (retval != EX_OK)
+		/*
+		 * If you want to continue with the next host,
+		 * the below should be 'return EX_TEMPFAIL'.
+		 */
+		break;		/* try another host address */
+	      return EX_OK;
 	  case EX_TEMPFAIL:
-	    retval = EX_TEMPFAIL;
-	    break;
+	      retval = EX_TEMPFAIL;
+	      break;
 	  }
 	}
 	return retval;
@@ -1640,10 +1647,14 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 	u_short p;
 	int errnosave;
 	char *se;
+	time_t now = time(NULL);
 
 	strcpy(ipaddress, dottedquad((struct in_addr *)ipaddrcp));
 	if (conndebug)
 	  fprintf(stderr, "Trying %s [%s] ... ", hostname, ipaddress);
+	if (logfp)
+	  fprintf(logfp, "%d#\t(Connecting to `%s' [%s] %24.24s)\n",
+		  pid, hostname, ipaddress, ctime(&now));
 	/* try grabbing a port */
 	sad.sin_family = AF_INET;
 	sad.sin_addr.s_addr = 0;
@@ -1655,6 +1666,8 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 	    fprintf(stderr, "%s\n", remotemsg);
 	  if (verboselog)
 	    fprintf(verboselog,"%s\n",remotemsg);
+	  if (logfp)
+	    fprintf(logfp,"%d#\t(Internal error, socket: %s)\n",pid,se);
 	  return EX_TEMPFAIL;
 	}
 	if (wantreserved && getuid() == 0) {
@@ -1670,6 +1683,8 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 		fprintf(verboselog,"%s\n", remotemsg);
 	      if (conndebug)
 		fprintf(stderr, "%s\n", remotemsg);
+	      if (logfp)
+		fprintf(logfp, "%d#\t(Internal error, bind: %s)\n", pid, s);
 	      return EX_UNAVAILABLE;
 	    }
 	  }
@@ -1680,6 +1695,8 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 	      fprintf(stderr, "%s\n", remotemsg);
 	    if (verboselog)
 	      fprintf(verboselog,"%s\n",remotemsg);
+	    if (logfp)
+	      fprintf(logfp,"%d#\t(Internal error, too many busy ports)\n", pid);
 	    return EX_TEMPFAIL;
 	  }
 	}
@@ -1712,6 +1729,8 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 		fprintf(stderr, "%s\n", remotemsg);
 	if (verboselog)
 		fprintf(verboselog,"%s\n",remotemsg);
+	if (logfp)
+	  fprintf(logfp,"%d#\t(connect failed to `%s' [%s]: %s)\n", pid, hostname, ipaddress, se);
 	switch (errnosave) {	/* from sendmail... */
 	case EISCONN:
 	case ETIMEDOUT:
@@ -1737,7 +1756,8 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 }
 
 SIGNAL_TYPE
-sig_pipe()
+sig_pipe(sig)
+int sig;
 {
 	if (logfp != NULL) {
 	  fprintf(logfp, "%d#\t*** Received SIGPIPE!\n", pid);
@@ -1747,7 +1767,8 @@ sig_pipe()
 }
 
 SIGNAL_TYPE
-sig_alarm()
+sig_alarm(sig)
+int sig;
 {
 	gotalarm = 1;
 }
@@ -1827,7 +1848,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	    fclose(smtpfp[0]); smtpfp[0] = NULL;
 	    fclose(smtpfp[1]); smtpfp[1] = NULL;
 	    if (logfp)
-	      fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
+	      fprintf(logfp, "%d#\t(closed SMTP channel - timeout on smtpwrite())\n", pid);
 	    smtp_isopen = 0;
 	    return EX_TEMPFAIL;
 	  } else if (r != len) {
@@ -1838,7 +1859,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	    fclose(smtpfp[0]); smtpfp[0] = NULL;
 	    fclose(smtpfp[1]); smtpfp[1] = NULL;
 	    if (logfp)
-	      fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
+	      fprintf(logfp, "%d#\t(closed SMTP channel - second timeout on smtpwrite() )\n", pid);
 	    smtp_isopen = 0;
 	    return EX_TEMPFAIL;
 	  }
@@ -1960,11 +1981,12 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	    fclose(smtpfp[0]); smtpfp[0] = NULL;
 	    fclose(smtpfp[1]); smtpfp[1] = NULL;
 	    if (logfp)
-	      fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
+	      fprintf(logfp, "%d#\t(closed SMTP channel - bad responce on smtpwrite() )\n", pid);
 	    smtp_isopen = 0;
 	    return EX_TEMPFAIL;
 	  } else {
-	    sprintf(remotemsg, "500 (Server hung up on us! Cmd: %s)",strbuf);
+	    sprintf(remotemsg, "500 (Server hung up on us! Cmd: %s)",
+		    strbuf == NULL ? "(null cmd)" : strbuf);
 	    notaryreport(NULL,NULL,remotemsg);
 	    dflag = 0;
 	    if (verboselog)
@@ -1972,7 +1994,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	    fclose(smtpfp[0]); smtpfp[0] = NULL;
 	    fclose(smtpfp[1]); smtpfp[1] = NULL;
 	    if (logfp)
-	      fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
+	      fprintf(logfp, "%d#\t(closed SMTP channel - hangup on smtpwrite() )\n", pid);
 	    smtp_isopen = 0;
 	    return EX_TEMPFAIL;
 	  }
@@ -1990,7 +2012,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	  fclose(smtpfp[0]); smtpfp[0] = NULL;
 	  fclose(smtpfp[1]); smtpfp[1] = NULL;
 	  if (logfp)
-	    fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
+	    fprintf(logfp, "%d#\t(closed SMTP channel - responce overrun on smtpwrite() )\n", pid);
 	  smtp_isopen = 0;
 	  return EX_TEMPFAIL;
 	}
@@ -2004,7 +2026,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	  fclose(smtpfp[0]); smtpfp[0] = NULL;
 	  fclose(smtpfp[1]); smtpfp[1] = NULL;
 	  if (logfp)
-	    fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
+	    fprintf(logfp, "%d#\t(closed SMTP channel - unexpected responce on smtpwrite() )\n", pid);
 	  smtp_isopen = 0;
 	  return EX_TEMPFAIL;
 	}
@@ -2459,7 +2481,6 @@ getrrtype(host, hbsize, rrtype, cnamelevel)
 	u_short type;
 	msgdata nbuf[BUFSIZ];
 	int first;
-	char *cnamestr = NULL;
 
 	qlen = res_mkquery(QUERY, host, C_IN, rrtype, (char *)NULL, 0, NULL,
 			   (char *)&buf, sizeof(buf));
